<%
'===============================================
' CactuShop ASP Shopping Cart
' 1999-2007 Cactusoft International FZ-LLC
' www.cactusoft.com
'===============================================
' All rights reserved.							
' Use of this code is covered by the terms and	
' conditions in the license agreement. No		
' unauthorized duplication or distribution is	
' permitted. Cactusoft's copyright notices must	
' remain in the ASP sections of the code.		
'===============================================

'-----------------------------------------------
'OPTION GROUP CLASS
'Contains an option group, and the selected option
'-----------------------------------------------
Class OptionGroup

	'----------------------------------------------------------
	'PUBLIC VARIABLES
	'----------------------------------------------------------
	Public OptionGroupID			'ID of the option group
	Public OptionGroupName			'Name of the option group
	Public OptionID					'ID of the option selected
	Public OptionName				'Name of the option selected
	Public PriceModifier			'Modification to the base price
	Public WeightModifier			'Modification to the base weight
	Public CheckBoxValue			'This either contains blank (not a checkbox), "y" (checkbox selected) or "n" (not selected)
	
End Class
	


'-----------------------------------------------
'BASKETITEM CLASS
'Holds information for one item in the basket
'All values in the basket are presumed converted
'to the users currency
'-----------------------------------------------
Class BasketItem

	'----------------------------------------------------------
	'PUBLIC VARIABLES
	'----------------------------------------------------------
	'Base values (stored in session)
	Public m_VersionID					'The version ID
	Public ComboVersionID				'The ID of the combination version
	Private m_quantity					'The quantity in the basket
	
	'Promotion values
	Public m_appliedpromo				'The number of promotion already applied for this version
	
	'Database values
	Public CodeNumber					'The version codenumber
	Public VersionName					'The name of the version
	Public ProductID					'The ID of the product
	Public ProductName					'The name of the product this is related to
	Public ProductType					'Product type - 's' for single, 'm' for multiple version, 'o' for options
	
	'Basket values
	Private m_extax						'The price of the item, before tax
	Private m_taxrate					'The tax rate applied to this item
	Public FreeShipping					'Whether the item has free shipping
	Public Weight						'Weight of a single item
	Public RRP							'The RRP of the version
	
	Public ApplyTax						'Whether the tax is on or off
	Public PricesIncTax					'Whether prices are entered incl or ex of tax
	
	Public OptionGroups()				'Array of option group classes
	Public HasCombinations				'Whetherh the options have specific combinations
	

	Public Sub SetOptionGroupArray(numSize)
		Redim OptionGroups(numSize)
	End Sub
		
	Public Function OptionGroupsList()
		For j = 0 to UBound(OptionGroups)
			OptionGroupsList = IIf(OptionGroupsList = "", OptionGroups(j).OptionGroupID, OptionGroupsList & "|" & OptionGroups(j).OptionGroupID)
		Next
	End Function
	
	Public Function OptionsList()
		For j = 0 to UBound(OptionGroups)
			OptionsList = IIf(OptionsList = "", OptionGroups(j).OptionID, OptionsList & "|" & OptionGroups(j).OptionID)
		Next	
	End Function
	
	
	Public Function OptionText(strType)
		If HasOptions Then
			Select Case strType
				Case "db": strBreak = vbcrlf
				Case "html": strBreak = "<br />"
				Case "mail": strBreak = vbcrlf & " "
			End Select
			For j = 0 to uBound(OptionGroups)
				With OptionGroups(j)
					If .CheckBoxValue = "" then				
						'Normal value - display
						OptionText = OptionText & IIf(.OptionGroupName = "", "", .OptionGroupName & ": ") & .OptionName
					Else
						'It's a checkbox - if it exists, show the option name only, otherwise skip
						If .CheckBoxValue = "y" then
							OptionText = OptionText & .OptionName
						End If
					End If
					
					'Add on break if this isn't the last one
					If j < ubound(OptionGroups) and OptionText <> "" then OptionText = OptionText & strBreak
				End With
			Next
		End If
	End Function
	
	
	Public Function HasOptions()
		'The item has options if the first option group ID > 0
		HasOptions = (OptionGroups(0).OptionGroupID > 0)
	End Function
	
	'Exposure of our private quantity.
	Public Property Let Quantity(value)
		m_quantity = NumSafe(value)
		If (GetAppVar("fractionalbasketquantities") <> "y") then m_quantity = CLng(m_quantity)
		If m_quantity > 32767 then m_quantity = 32767
	End Property
	Public Property Get Quantity
		Quantity = m_quantity
	end Property
	
		'Exposure of our private applied promo quantity.
	Public Property Let AppliedPromo(value)
		m_appliedpromo = NumSafe(value)
		If (GetAppVar("fractionalbasketquantities") <> "y") then m_applied = CLng(m_appliedpromo)
		If m_appliedpromo > 32767 then m_appliedpromo = 32767
	End Property
	
	Public Property Get AppliedPromo
		AppliedPromo = m_appliedpromo
	end Property
	
	'Increases the quantity by value
	Public Sub IncreaseQuantity(value)
		Quantity = m_quantity + value
	End Sub

	'Increases the quantity by value
	Public Sub DecreaseQuantity(value)
		Quantity = m_quantity - value
	End Sub

	'Returns querystring information when linking to a specific version
	Public Function GetLinkQS
		GetLinkQS = "P_ID=" & ProductID & "&amp;V_ID=" & VersionID & "&amp;" & IIf(HasOptions, "strOptions=" & Replace(OptionsList, "|", ",") & "&amp;", "")
	End Function
	

	'----------------------------------------------------------
	'INITIALISE - Called when an instance of the class is created
	'----------------------------------------------------------
	Private Sub Class_Initialize()
		ApplyTax = true
		PriceIncTax = true
		HasCombinations = false
		m_ComboVersion = 0
		VersionID = 0
	End Sub
		
	
	'----------------------------------------------------------
	'CALCULATED VALUES 
	'----------------------------------------------------------
	'Returns a combination of product and version
	Public Property Let VersionID(value)
		m_versionID = value
	End Property
	
	Public Property Get VersionID
		If ComboVersionID > 0 then
			VersionID = ComboVersionID
		Else
			VersionID = m_versionID
		End If
	End Property
	
	Public Property Get Name
		Name = ProductName & " - " & VersionName
	End Property
	
	'ExTax
	Public Property Let ExTax(value)
		m_extax = value
	End Property
	Public Property Get ExTax
		ExTax = SafeRound(m_extax, 2)
	End Property
	
	'IncTax - returns the inc tax amount - ex tax + tax amount
	'Uses the base - unrounded -mmount of extax otherwise we
	'get rounding problems
	Public Property Get IncTax
		IncTax = SafeRound(m_extax + TaxAmount, 2)
	End Property
	
	'TaxAmount - returns the total amount of tax
	Public Property Get TaxAmount
		If Not ApplyTax then
			TaxAmount = 0
		Else
			TaxAmount = (ExTax * TaxRate)
		End If
	End Property
	
	'TaxAmount - returns the total amount of tax
	Public Property Let TaxRate(value)
		m_taxrate = value
	End Property
	Public Property Get TaxRate
		If Not ApplyTax then
			TaxRate = 0
		Else
			TaxRate = SafeRound(m_taxrate, 4)
		End If
	End Property	
	
	'Inc & ex tax without rounding
	Public Property Get ExTaxNoRound
		ExTaxNoRound = m_exTax
	End Property
	
	Public Property Get IncTaxNoRound
		IncTaxNoRound = m_exTax * (TaxRate + 1)
	End Property
		

	
	'----------------------------------------------------------
	'ROW TOTALS
	'multiplications of base or calculated values for row totals.
	'We use the private local variables here, as exposed public
	'properties are rounded. 
	'INC TAX: Round per item.
	'EX TAX: Round per row
	'----------------------------------------------------------
	'RowTotal - quantity x price ex tax
	Public Property Get RowExTax
		If PricesIncTax Then
			RowExTax = SafeRound(ExTax * Quantity, 2)
		Else
			RowExTax = SafeRound(ExTaxNoRound * Quantity, 2)
		End If
	End Property
	
	'The total price incuding tax on this row
	Public Property Get RowIncTax
		If PricesIncTax then
			RowIncTax = SafeRound(IncTax * Quantity, 2)
		Else
			RowIncTax = SafeRound(IncTaxNoRound * Quantity, 2)
		End If
	End Property
	
	'The amount of tax to add for this row
	Public Property Get RowTaxAmount
		If Not ApplyTax Then
			RowTaxAmount = 0
		Else
			If PricesIncTax then
				RowTaxAmount = SafeRound(RowIncTax - RowExTax, 2)
			Else
				RowTaxAmount = IncTaxNoRound * Quantity - RowExTax
			End If
		End If
	End Property
	
	'RowWeight - returns the total weight for the row
	Public Property Get RowWeight
		RowWeight = Weight * Quantity
	End Property
	
	'RowAmountSaved - the amount you save (comparison with RRP) for the whole row
	Public Property Get RowAmountSaved
		If PricesIncTax Then
			RowAmountSaved = Max(SafeRound(Quantity * (RRP - IncTax), 2), 0)
		Else
			RowAmountSaved = Max(SafeRound(Quantity * (RRP - ExTax), 2), 0)
		End If
	End Property
	
	
	'----------------------------------------------------------
	'INVOICE ROW RELATED
	'----------------------------------------------------------
	'IR_PricePerItem - returns the price-per-item for the 
	'invoicerows table.
	Public Property Get IR_PricePerItem
		If PricesIncTax then
			IR_PricePerItem = IncTax
		Else
			IR_PricePerItem = ExTax
		End If
	End Property
	
	'IR_TaxPerItem - Returns the tax-per-item for the 
	'invoicerows table
	Public Property Get IR_TaxPerItem
		If PricesIncTax then
			IR_TaxPerItem = TaxAmount
		Else
			IR_TaxPerItem = TaxRate
		End If
	End Property
	
	
End Class



'-----------------------------------------------
'BASKET MODIFIER
'Contains details to a 'basket modifier' - such
'as an additional charge, a discount etc.
'-----------------------------------------------
Class BasketModifier
	
	Private m_exTax
	Private m_incTax
	Private m_taxRate		'may not be applicable
	
	Public Property Get ExTax
		ExTax = m_exTax
	end Property
	Public Property Let ExTax(value)
		m_exTax = value
	End Property
	
	Public Property Get IncTax
		IncTax = m_incTax
	end Property
	Public Property Let IncTax(value)
		m_incTax = value
	End Property
	
	Public Property Get TaxRate
		TaxRate = SafeRound(m_taxRate, 4)
	end Property
	Public Property Let TaxRate(value)
		m_taxRate = value
	End Property
	
	Public Property Get TaxAmount
		TaxAmount = m_incTax - m_exTax
	End Property
	
End Class


'-----------------------------------------------
'BASKET CLASS
'Represents a 'basket' of multiple basket items
'Contains an 'items' array, plus various functions
'to access this data
'-----------------------------------------------
CONST BASKET_ARRAY_STEP = 5

CONST SHIPPING_TYPE_PICKUP = "p"
CONST SHIPPING_TYPE_FREE = "f"		'this can be all electronic products, or all 'free shipping' products
CONST SHIPPING_TYPE_DISABLED = "d"
CONST SHIPPING_TYPE_UPS = "u"
CONST SHIPPING_TYPE_CACTUS = "c"

Class Basket
	'----------------------------------------------------------
	'PRIVATE VARIABLES
	'----------------------------------------------------------
	Private m_items()						'Our private array of basketitems
	Private m_lastindex						'Our private property for the last index with an item in
	
	Private m_totalitems					'Total items in teh basket
	Private m_allfreeshipping				'Whether all items in the basket are free from shipping
	Private m_hasfreeshipping				'Whether at least one item is free from shipping
	
	Private m_totalweight					'Total weight of the basket
	Private m_totalextax					'Total basket value, ex tax
	Private m_totaltaxamount				'total basket tax amount
	
	Private m_shippingtotalweight			'Total weight of basket items that are shipped
	Private m_shippingtotalextax			'Total ex tax ''	...		''		....
	Private m_shippingtotaltaxamount		'Total tax amount ''	...	''		...
	Private m_shippingType					'Internal variable for the type of shipping chosen
	
	Private m_shippingName					'Name for chosen shipping method
	Private m_shippingDescription			'Description for chosen shipping method
	
	Private m_applytax						'Whether tax is enabled or disabled. Dependant on users shipping
	Private m_pricesinctax					'Whether prices are entered inc or ex tax
	
	Private m_adjustedQuantities			'Comma-seperated list of versions that have had quantities adjusted due to stock levels
	Private m_adjustedForElectronic			'Whether it's been adjusted due to an electronic product having a qty > 1
	
	Private m_totalAmountSaved				'Total 'saved' in the basket based on the RRP field
	
	Private m_customerDiscountPercentage	'Discount % for customer. May include include their customer group discount too.

	Private m_DTax							'Variable to store D_Tax value is USmultistatetax is activated.

	'Basket modifiers
	Public ShippingPrice
	Public OrderHandlingPrice
	Public CouponDiscount
	Public CustomerDiscount
	Public PromotionDiscount
	
	'Array for promotions and counter
	Public Promotions
	Public PromotionsCount
	
	Public Property Get D_Tax
		D_Tax = m_DTax
	End Property
	
	Public Property Let D_Tax(value)
		m_DTax = value
	End Property			
				
	Public Property Let ApplyTax(value)
		If m_applytax <> value then
			m_applytax = value
			
			'Loop through all basket items and set their tax setting
			For i = 0 to m_lastindex
				m_items(i).ApplyTax = m_applytax
			Next
			
			'Rebuild
			CalculateTotals
		End If
	End Property
	Public Property Get ApplyTax
		ApplyTax = m_applytax
	End Property
		
	Public Property Let PricesIncTax(value)
		If m_pricesinctax <> value then
			m_pricesinctax = value
			
			'Loop through all basket items and set their tax setting
			For i = 0 to m_lastindex
				m_items(i).PricesIncTax = m_pricesinctax
			Next
			
			'Rebuild
			CalculateTotals
		End If
	End Property
	Public Property Get PricesIncTax
		PricesIncTax = m_pricesinctax
	End Property
	
	
	'----------------------------------------------------------
	'TOTAL PROPERTIES
	'Most just return the private variables. The CalculateTotals
	'function actually does the compiling of these values.
	'Those with 'shipping' are items that aren't exempt from 
	'shipping (downloadable, free shipping). Some are calculated
	'field based on privat evariables
	'----------------------------------------------------------
	'TotalItems - the total number of versions in the basket
	Public Property Get TotalItems
		TotalItems = m_totalitems
	End Property
	

	'HasFreeShipping - Returns whether at least one item has free shipping
	Public Property Get HasFreeShipping
		HasFreeShipping = m_hasfreeshipping
	End Property
	
	Public Property Get AllFreeShipping
		AllFreeShipping = m_allfreeshipping
	End Property
		
	'TotalWeight - Total weight of items in the basket
	Public Property Get TotalWeight
		TotalWeight = m_totalweight
	End Property
	
	'Total value of basket, ex- and inc-tax
	Public Property Get TotalExTax
		TotalExTax = m_totalextax
	End Property
	
	Public Property Get TotalTaxAmount
		TotalTaxAmount = m_totaltaxamount
	End Property
	
	Public Property Get TotalIncTax
		TotalIncTax = TotalExTax + TotalTaxAmount
	End Property
	
	Public Property Get TotalTaxRate
		If TotalExTax = 0 then
			TotalTaxRate = 0
		Else
			TotalTaxRate = SafeRound(TotalTaxAmount  / TotalExTax, 4)
		End If
	End Property
	
	'Total amount saved - difference between price and rrp. Don't
	'let this go lower than zero
	Public Property Get TotalAmountSaved
		TotalAmountSaved = Max(m_totalamountsaved, 0)
	End Property
	
	'ShippingTotalWeight - Total weight of shipped items in the basket
	Public Property Get ShippingTotalWeight
		ShippingTotalWeight = m_shippingtotalweight
	End Property
	
	'Shipping total - amount in the basket that should be used when calculating
	'the cost of shipping (i.e. no downloadable or free-shipping products).
	Public Property Get ShippingTotalExTax
		ShippingTotalExTax = m_shippingtotalextax
	End Property
	
	Public Property Get ShippingTotalTaxAmount
		ShippingTotalTaxAmount = m_shippingtotaltaxamount
	End Property
	
	Public Property Get ShippingTotalIncTax
		ShippingTotalIncTax = ShippingTotalExTax + ShippingTotalTaxAmount
	End Property	
	
	'AdjustedQuantities - comma-sep list of adjusted quantities. Also a boolean
	'flag to see if amounts were adjusted because they are electronic
	Public Property Get AdjustedQuantities
		AdjustedQuantities = m_adjustedQuantities
	End Property 
	Public Property Get AdjustedForElectronic
		AdjustedForElectronic = m_adjustedForElectronic
	End Property 		
		
	'Shipping
	Public Property Get ShippingName
		ShippingName = m_shippingName
	End Property
	
	Public Property Get ShippingDescription
		ShippingDescription = m_shippingDescription
	End Property

	'Customer discount - calculated on the fly based on m_customerDiscountPercentage
	Public Property Get CustomerDiscountPercentage
		CustomerDiscountPercentage = m_customerDiscountPercentage
	End Property
	
	'Cost of order after discounts
	Public Property Get TotalDiscountPriceIncTax
		TotalDiscountPriceIncTax = TotalIncTax + CouponDiscount.IncTax + PromotionDiscount.IncTax + CustomerDiscount.IncTax
	End Property
	
	Public Property Get TotalDiscountPriceExTax
		TotalDiscountPriceExTax = TotalExTax + CouponDiscount.ExTax + PromotionDiscount.ExTax + CustomerDiscount.ExTax
	End Property
	
	Public Property Get TotalDiscountPriceTaxAmount
		TotalDiscountPriceTaxAmount = TotalDiscountPriceIncTax - TotalDiscountPriceExTax
	End Property
		
	Public Property Get TotalDiscountPriceTaxRate
		If TotalDiscountPriceExTax = 0 then
			TotalDiscountPriceTaxRate = 0
		Else
			TotalDiscountPriceTaxRate = TotalDiscountPriceTaxAmount  / TotalDiscountPriceExTax
		End If
	End Property
	
	
	'Final total cost of order - with discounts, shipping and order handling
	Public Property Get FinalPriceIncTax
		FinalPriceIncTax = SafeRound(TotalDiscountPriceIncTax + ShippingPrice.IncTax + OrderHandlingPrice.IncTax, 2)
		if FinalPriceIncTax < 0 then FinalPriceIncTax = 0
	End Property
	
	Public Property Get FinalPriceExTax
		FinalPriceExTax = SafeRound(TotalDiscountPriceExTax + ShippingPrice.ExTax + OrderHandlingPrice.ExTax, 2)
		if FinalPriceExTax < 0 then FinalPriceExTax = 0
	End Property
	
	Public Property Get FinalPriceTaxAmount
		FinalPriceTaxAmount = FinalPriceIncTax - FinalPriceExTax
	End Property
				
	'Total value to attribute the affiliate (no discounts, shipping, tax etc).
	'Basically the raw ex-tax value, converted to the default currency rate
	Public Property Get TotalValueToAffiliate
		TotalValueToAffiliate = SafeRound(TotalExTax / objCurrency.ExchangeRate, 2)
	End Property
	
	
	'Final gateway price - convert to gateway currency
	Public Property Get FinalGatewayPriceIncTax
		FinalGatewayPriceIncTax = Round((FinalPriceIncTax / objCurrency.ExchangeRate) * objPayment.GatewayCurrency.ExchangeRate, 2)
	End Property
	
	'----------------------------------------------------------
	'SHIPPING TYPE
	'Exposes variable to variable boolean values
	'----------------------------------------------------------
	Public Property Get FreeShipping
		FreeShipping = (m_shippingType = SHIPPING_TYPE_FREE)
	End Property
	
	Public Property Get ShippingPickup
		ShippingPickup = (m_shippingType = SHIPPING_TYPE_PICKUP)
	End Property
	
	Public Property Get ShippingUPS
		ShippingUPS = (m_shippingType = SHIPPING_TYPE_UPS)
	End Property
	
	Public Property Get ShippingCactus
		ShippingCactus = (m_shippingType = SHIPPING_TYPE_CACTUS)
	End Property
	
	Public Property Get ShippingDisabled
		ShippingDisabled = (m_shippingType = SHIPPING_TYPE_DISABLED)
	End Property
	
	
	'----------------------------------------------------------
	'INITIALISE - Called when an instance of the class is created
	'----------------------------------------------------------
	Private Sub Class_Initialize()
		Redim m_items(BASKET_ARRAY_STEP)
		m_lastindex = -1
		m_applytax = true
	End Sub
	
	'Loads in the basket through two comma-sep strings for items and qtys
	Public Function Load(strItems, strQuantities, strOptionGroups, strOptions)
		Dim aryItems, aryQuantities, i, numItem, numQuantity
		
		'Split values in to array
		aryItems = Split(strItems & "", ",")
		aryQuantities = Split(strQuantities & "", ",")
		aryOptionGroups = Split(strOptionGroups & "", ",")
		aryOptions = Split(strOptions & "", ",")
				
		'Loop through the items
		For i = 0 to UBound(aryItems)
			'Check we've got the quantity, option group and options in the array
			If Ubound(aryQuantities) >= i and Ubound(aryOptionGroups) >= i and Ubound(aryOptions) >= i then
				'Get the value of the item and quantity
				numItem = NumSafe(aryItems(i))
				numQuantity = NumSafe(aryQuantities(i))
				strItemOptionGroups = Trim(aryOptionGroups(i))
				strItemOptions = Trim(aryOptions(i))
				
				'Add the values if item and quantity > 0
				If numItem > 0 and numQuantity > 0 then
					If Not Exists(numItem, strItemOptions) then
						Add numItem, numQuantity, strItemOptionGroups, strItemOptions
					End If
				End If
			End If
		Next
		
		'Create basket modifiers
		Set OrderHandlingPrice = New BasketModifier
		Set ShippingPrice = New BasketModifier
		Set PromotionDiscount = New BasketModifier
		Set CouponDiscount = New BasketModifier
		Set CustomerDiscount = New BasketModifier		
		
	End Function	
	
	'Validates the data loaded in to the basket. 
	Public Function Validate(objRS, blnAllowOutOfStock)
			
		'Set variables from application
		m_adjustedForElectronic = false
		PricesIncTax = (GetAppVar("pricesinctax") = "y")		
		blnAllowPurchaseOutOfStock = (GetAppVar("allowpurchaseoutofstock") = "y") or blnAllowOutOfStock
		blnOnlyOneDownload = (GetAppVar("onlyonedownload") = "y")
		
		'Only proceed if we have basket items
		If Count > 0 then
			
			'Define the select and join part to pull out a version
			strSelectSQL = "P_ID, P_Type, T_Taxrate, V_Price, V_Weight, V_RRP, V_ID, P_Name" & CStr(numLanguageID) & ", V_Name" & CStr(numLanguageID) & ", V_CodeNumber, V_Live, P_Live, CAT_Live, V_Quantity, V_QuantityWarnLevel, V_DownloadType"
			strJoinSQL = "(((((tblCactuShop" & TABLE_PREFIX & "Versions INNER JOIN tblCactuShop" & TABLE_PREFIX & "Products ON tblCactuShop" & TABLE_PREFIX & "Versions.V_ProductID = tblCactuShop" & TABLE_PREFIX & "Products.P_ID) INNER JOIN tblCactuShop" & TABLE_PREFIX & "ProductCategoryLink ON tblCactuShop" & TABLE_PREFIX & "Products.P_ID = tblCactuShop" & TABLE_PREFIX & "ProductCategoryLink.PCAT_ProductID) INNER JOIN tblCactuShop" & TABLE_PREFIX & "Categories ON tblCactuShop" & TABLE_PREFIX & "ProductCategoryLink.PCAT_CategoryID = tblCactuShop" & TABLE_PREFIX & "Categories.CAT_ID) INNER JOIN tblCactuShop" & TABLE_PREFIX & "TaxRates ON tblCactuShop" & TABLE_PREFIX & "Versions.V_Tax = tblCactuShop" & TABLE_PREFIX & "TaxRates.T_ID) LEFT OUTER JOIN tblCactuShop" & TABLE_PREFIX & "Suppliers ON tblCactuShop" & TABLE_PREFIX & "Products.P_SupplierID = tblCactuShop" & TABLE_PREFIX & "Suppliers.SUP_ID)"
			strWhereSQL = "P_Live='y' AND V_Live='y' AND CAT_Live='y' AND V_CustomerGroupID IN (0," & NumSafe(DB_CG_ID) & ") AND P_CustomerGroupID IN (0," & NumSafe(DB_CG_ID) & ") AND CAT_CustomerGroupID IN (0," & NumSafe(DB_CG_ID) & ") AND (SUP_Live <> 'n' OR SUP_Live IS NULL) AND (P_Name" & CStr(numLanguageID) & " <> '')"
			
			'We may need a new recordset if we're doing customer
			'group prices or option groups
			Set objRSPrices = Server.CreateObject("ADODB.Recordset")
						
			'Create a dictionary to keep count of version stock levels in the basket.
			Set objVersionQuantitiesDict = Server.CreateObject("Scripting.Dictionary")
			
			'Loop through all basket records
			for i = 0 to Count - 1
				With Item(i)
					'If we've got options, try and pull out the version combination. 
					'If this exists, then update the ID so it points to this combo. 
					blnGotCombination = false
					strOptionsJoinSQL = "": strOptionsWhereSQL = "": strProdOptionsJoinSQL = "": strProdOptionsWhereSQL = ""
					strBracketSQL = ""
					
					If .HasOptions then
						
						'Loop through all options
						For numOptionCount = 0 to UBound(.OptionGroups)
							'Build up the query - this joins on to each option selection
							strBracketSQL = strBracketSQL & "("
							strOptionsJoinSQL = strOptionsJoinSQL & " INNER JOIN tblCactuShop" & TABLE_PREFIX & "VersionOptionLink AS opt" & numOptionCount & " ON tblCactuShop" & TABLE_PREFIX & "Versions.V_ID = opt" & numOptionCount & ".V_OPT_VersionID)"
							strOptionsWhereSQL = strOptionsWhereSQL & " AND opt" & numOptionCount & ".V_OPT_OptionID = " & .OptionGroups(numOptionCount).OptionID
							
							strProdOptionsJoinSQL = strProdOptionsJoinSQL & " INNER JOIN tblCactuShop" & TABLE_PREFIX & "ProductOptionLink AS opt" & numOptionCount & " ON p.P_ID = opt" & numOptionCount & ".P_OPT_ProductID)"
							strProdOptionsWhereSQL = strProdOptionsWhereSQL & " AND opt" & numOptionCount & ".P_OPT_OptionID = " & .OptionGroups(numOptionCount).OptionID
						Next
						
						'Pull out the product - this ensures that pulling out the
						'right combination is limited to the product we're in. No security
						'required here (we'll deal with that when we get the version). We join
						'on to option groups to check we've got the right number of combinations
						'and on to product options to check the ones we're joined to are allowed
						strQuery = "SELECT P_ID, P_OPTG_OptionGroupID FROM " & strBracketSQL & "((tblCactuShop" & TABLE_PREFIX & "Products AS p INNER JOIN tblCactuShop" & TABLE_PREFIX & "Versions AS v ON p.P_ID = v.V_ProductID) INNER JOIN tblCactuShop" & TABLE_PREFIX & "ProductOptionGroupLink opgl ON p.P_ID = opgl.P_OPTG_ProductID)" & strProdOptionsJoinSQL & " WHERE v.V_ID=" & .VersionID & strProdOptionsWhereSQL
						Call ExecuteRS(strQuery, objRS)
						If objRS.RecordCount = 0 or objRS.RecordCount <> UBound(.OptionGroups) + 1 then
							.Quantity = 0
							objRS.Close
						Else
							numOptionProductID = objRS("P_ID")
							objRS.Close
							'Try and pull out the version
							If numOptionCount > 0 then
								strQuery = "SELECT DISTINCT " & strSelectSQL & " FROM " & strBracketSQL & strJoinSQL & " " & strOptionsJoinSQL & " WHERE " & strWhereSQL & strOptionsWhereSQL & " AND P_ID=" & numOptionProductID
								Call ExecuteRS(strQuery, objRS)
								If objRS.RecordCount > 1 then
									'Got more than one - they haven't got enough selected. Set quantity to 0
									objRS.Close
									.Quantity = 0
								
								ElseIf objRS.RecordCount = 1 then
									.HasCombinations = true
									.ComboVersionID = objRS("V_ID")
								Else
									objRS.Close
									'Check to see if we *should* have a combination - we can obtain that
									'by trying to pull out a version with V_Type of 'c'
									strQuery = "SELECT COUNT(*) FROM tblCactuShop" & TABLE_PREFIX & "Versions v INNER JOIN tblCactuShop" & TABLE_PREFIX & "Products p ON v.V_ProductID = p.P_ID WHERE p.P_ID = " & numOptionProductID & " AND v.V_Type='c'"
									If clng(ExecuteScalar(strQuery)) > 0 then
										.Quantity = 0
										
									End If
								end if
							End If
						End If
					End If
					
					'Otherwise, pull out the version information. 
					If not .HasCombinations then
						strQuery = "SELECT " & strSelectSQL & " FROM " & strJoinSQL & " WHERE " & strWhereSQL & " AND V_ID=" & .VersionID
						Call ExecuteRS(strQuery, objRS)
					End If
				
					
					'If we don't have the record, set quantity to 0
					If objRS.BOF and objRS.EOF then
						.Quantity = 0
						
					ElseIf objRS("P_Type") = "o" and not .HasOptions then
						'Item needs to have option choices
						.Quantity = 0
													
					Else
					
						'Set basic database values
						.CodeNumber = objRS("V_CodeNumber")
						.VersionName = objRS("V_Name" & Cstr(numLanguageID))
						.ProductID = objRS("P_ID")
						.ProductName = objRS("P_Name" & Cstr(numLanguageID))
						.ProductType = objRS("P_Type")
						.RRP = objRS("V_RRP") * objCurrency.ExchangeRate
						
						'Set weight and tax
						numWeight = objRS("V_Weight")
						
						'If D_Tax is set then apply it in the taxrate
						if not objBasket.D_Tax <> "" then .TaxRate = objRS("T_Taxrate")/100 else .TaxRate = (objRS("T_Taxrate")/100) * objBasket.D_Tax
						
						'Set whether this item has free shipping
						V_DownloadType = objRS("V_DownloadType") & ""
						.FreeShipping = Not (V_DownloadType = "" or V_DownloadType = "n")
							
						'If the item is electronic, one download is allowed, and the user has
						'more than one for quantity, change quantity and warn user
						If blnOnlyOneDownload and (V_DownloadType = "l" or V_DownloadType = "u") and .Quantity > 1 then
							.Quantity = 1
							m_adjustedQuantities = IIf(m_adjustedQuantities = "", .VersionID & "-" & .OptionsList, m_adjustedQuantities & "," & .VersionID & "-" & .OptionsList)
							m_adjustedForElectronic = true
						End If
								
						'Adjust quantities if stock level too low
						If not blnAllowPurchaseOutOfStock then
							'Only warn if tracking turned on for this item (warn level
							'not set to 0)
							blnWarn = (objRS("V_QuantityWarnLevel") <> 0)
							If blnWarn then 
								'Add the quantity to our basket stock count for this version
								If objVersionQuantitiesDict.Exists(.VersionID) Then 
									numCurrentBasketQty = objVersionQuantitiesDict.Item(.VersionID)
								Else
									numCurrentBasketQty = 0
								End If
								numNewBasketQty = numCurrentBasketQty + .Quantity
								numMaxStockLevel = objRS("V_Quantity")
								
								If numNewBasketQty > numMaxStockLevel then
									'Set down the quantity. Make a note of adjusted quantities
									'so we can warn them on basket.asp. Make sure this doesn't go
									'below 0
									.Quantity = Max(numMaxStockLevel - numCurrentBasketQty, 0)
									numNewBasketQty = numCurrentBasketQty + .Quantity
									m_adjustedQuantities = IIf(m_adjustedQuantities = "", .VersionID & "-" & .OptionsList, m_adjustedQuantities & "," & .VersionID & "-" & .OptionsList)
								End If
								
								'Store the new stock level
								If objVersionQuantitiesDict.Exists(.VersionID) Then 
									objVersionQuantitiesDict.Item(.VersionID) = numNewBasketQty
								Else
									objVersionQuantitiesDict.Add .VersionID, numNewBasketQty
								End If
							End If
						End if

						'Get basic price, modify for customer group pricing (if lower)
						numPrice = objRS("V_Price")
						If SOFTWARE_IS_PRO Then
							If DB_CG_ID > 0 then
								numCustomerGroupPrice = GetCustomerGroupPriceForVersion(objRSPrices, DB_CG_ID, objRS("V_ID"))
								If numCustomerGroupPrice > 0 then
									numPrice = Min(numCustomerGroupPrice, numPrice)
								End If
							End If
						End If
						
						'Get price break level (if lower)
						If SOFTWARE_IS_PRO then
							numQuantityDiscountPrice = GetQuantityDiscountPriceForVersion(objRSPrices, .Quantity, .VersionID)
							If numQuantityDiscountPrice > 0 then
								numPrice = Min(numQuantityDiscountPrice, numPrice)
							End If
						End if
						
						
						'---------------------------------------------------
						'OPTIONS VALIDATION
						'---------------------------------------------------
						If .HasOptions Then
							'Pull out price/weight information for our optiongroups & options
							strQuery = "SELECT og.OPTG_ID, og.OPTG_Name" & CStr(numLanguageID) & ", o.OPT_ID, o.OPT_Name" & CStr(numLanguageID) & ", po.P_OPT_PriceChange, po.P_OPT_WeightChange, OPT_CheckBoxValue " &_
									   "FROM (tblCactuShop" & TABLE_PREFIX & "OptionGroups AS og INNER JOIN (tblCactuShop" & TABLE_PREFIX & "ProductOptionLink AS po INNER JOIN tblCactuShop" & TABLE_PREFIX & "Options AS o ON po.P_OPT_OptionID = o.OPT_ID) ON og.OPTG_ID = o.OPT_OptionGroupID) INNER JOIN (tblCactuShop" & TABLE_PREFIX & "Products AS p INNER JOIN tblCactuShop" & TABLE_PREFIX & "ProductOptionGroupLink AS pog ON p.P_ID = pog.P_OPTG_ProductID) ON (og.OPTG_ID = pog.P_OPTG_OptionGroupID) AND (po.P_OPT_ProductID = p.P_ID) " &_
									   "WHERE p.P_ID=" & .ProductID & " AND og.OPTG_ID In (" & Replace(.OptionGroupsList, "|", ",") & ") AND o.OPT_ID In (" & Replace(.OptionsList, "|", ",") & ") " &_
									   "ORDER BY pog.P_OPTG_OrderByValue, po.P_OPT_OrderByValue"
							Call ExecuteRS(strQuery, objRSPrices)
							
							'Loop through RS
							Do While Not objRSPrices.EOF 
								'Loop through the option groups and find the matching entry
								OPTG_ID = objRSPrices("OPTG_ID")
								For j = 0 to UBound(.OptionGroups)
									If .OptionGroups(j).OptionGroupID = OPTG_ID then
										With .OptionGroups(j)
											.OptionGroupName = objRSPrices("OPTG_Name" & Cstr(numLanguageID))
											.OptionID = objRSPrices("OPT_ID")
											.OptionName = objRSPrices("OPT_Name" & Cstr(numLanguageID))
											.PriceModifier = objRSPrices("P_OPT_PriceChange")
											.WeightModifier = objRSPrices("P_OPT_WeightChange")
											.CheckBoxValue = objRSPrices("OPT_CheckBoxValue") & ""
										End With
											
										'Adjust our running price and weight - only if we've not
										'got combinations though!
										If not .HasCombinations Then
											numPrice = numPrice + .OptionGroups(j).PriceModifier
											numWeight = numWeight + .OptionGroups(j).WeightModifier
										end if
										
										Exit for
									End If
								Next
								objRSPrices.MoveNext 
							Loop
							objRSPrices.Close 
						End If						
											
						'Modify price by the user-selected currency rate
						numPrice = SafeRound(objCurrency.Convert(numPrice), 2)
						
						'Calculate the ex-tax - this differs as numPrice will hold 
						'a different value depending on pricesinctax
						If PricesIncTax then
							.ExTax = numPrice * (1 / (1 + .TaxRate))
						Else
							.ExTax = numPrice
						End if
						
						
						
						'Set the weight
						.Weight = numWeight
						
					End if

					objRS.Close
				End With
			next
			Set objVersionQuantitiesDict = Nothing
			Set objRSPrices = Nothing
		End if	
	
	End Function
	
	
	'"Cleans" the array of zero-quantity items. In effect this rebuilds
	'it using the load function
	Public Function Clean()
		'Get local copies of our items and qtys
		Dim strItems, strQuantities
		strItems = Items
		strQuantities = Quantities
		strOptionGroups = OptionGroups
		strOptions = Options
		
		'Clear the array
		m_lastindex = -1
		Redim m_items(BASKET_ARRAY_STEP)
		
		'Rebuild with our items
		Load strItems, strQuantities, strOptionGroups, strOptions
		
	End Function
	
	'----------------------------------------------------------
	'ARRAY ACCESS METHODS
	'These methods open up access to the our private m_items array
	'----------------------------------------------------------
	'Item - returns an item in the array by index
	Public Property Get Item(index)
		Set Item = m_items(index)
	End Property
	
	'Returns an item based on the version id. Will return nothing 
	'if the item doesn't exist.
	Public Property Get Version(numVersionID, strOptionIDs)
		Dim i
		For i = 0 to m_lastindex
			If m_items(i).m_versionID = numVersionID then
				'Check the option list is the same
				If m_items(i).OptionsList & "" = Replace(strOptionIDs, ",", "|") & "" Then
					Set Version = m_items(i)
					Exit Property
				End If
			End If
		Next
	End Property	
	
	'Count - Gets the total number of items in the basket	
	Public Property Get Count
		Count = m_lastindex + 1
	End Property
	
	'Exists - Returns whether or not a version exists in the basket
	'Also checks that the option IDs are the same
	Public Property Get Exists(numVersionID, strOptionIDs)
		Dim i
		Exists = false
		For i = 0 to m_lastindex
			If m_items(i).m_versionID = numVersionID then
				'Check the option list is the same
				If m_items(i).OptionsList & "" = Replace(strOptionIDs, ",", "|") & "" Then
					Exists = true
					Exit Property
				End If
			End If
		Next
	End Property
	
	'Add - Adds a new item to the basket. Assumes it doesn't already exist.
	Public Function Add(VersionID, Quantity, strOptionGroups, strOptions)

		'Increase the number of items
		m_lastindex = m_lastindex + 1
		
		'If the array needs expanding, do it
		If m_lastindex > ubound(m_items) then
			Redim Preserve m_items(ubound(m_items) + BASKET_ARRAY_STEP)
		End If
		
		'Create the new basket item 
		Set m_items(m_lastindex) = New BasketItem
		With m_items(m_lastindex)
			'Set version and quantity
			.VersionID = NumSafe(VersionID)
			.Quantity = NumSafe(Quantity)
			
			'Set up option groups and options - split in to arrays
			aryOptionGroups = Split(strOptionGroups, "|")
			aryOptions = Split(strOptions, "|")
			
			'If the arrays aren't the same size, something's gone wrong -
			'set the quantity to 0 so they're not added
			If UBound(aryOptionGroups) <> UBound(aryOptions) Then
				.Quantity = 0
			Else
				'Create the array and store the IDs
				.SetOptionGroupArray(UBound(aryOptionGroups))
				For i = 0 to UBound(aryOptionGroups)
					'Ensure everything is numeric
					numOptionGroupID = NumSafe(aryOptionGroups(i))
					numOptionID = NumSafe(aryOptions(i))
					Set .OptionGroups(i) = New OptionGroup
					With .OptionGroups(i)
						.OptionGroupID = numOptionGroupID
						.OptionID = numOptionID
					End With
				Next
			End If
		End With
		
		'Return the new index
		Add = m_lastindex
		
	End Function
	
	'Delete - deletes an item from the basket (actually
	'it sets it's quantity to 0)
	Public Sub Delete(VersionID, strOptionIDs)
		Version(VersionID, strOptionIDs).Quantity = 0	
	End Sub
	
	
	'----------------------------------------------------------
	'BASKET STRING
	'Returns comma-sep string of items & quantites in the basket.
	'This is used to store in the session. It skips out items
	'that are on 0 quantity.
	'----------------------------------------------------------
	Public Property Get Items
		For i = 0 to m_lastindex
			If m_items(i).Quantity > 0 then
				'Uses the "private" m_VersionID to get the real ID, not the
				'combination version ID
				Items = IIf(Items = "", m_items(i).m_VersionID, Items & "," & m_items(i).m_VersionID)
			End If
		Next
	End Property
	
	Public Property Get Quantities
		For i = 0 to m_lastindex
			If m_items(i).Quantity > 0 then
				Quantities = IIf(Quantities = "", m_items(i).Quantity, Quantities & "," & m_items(i).Quantity)
			End If
		Next
	End Property
	
	Public Property Get OptionGroups
		For i = 0 to m_lastindex
			If m_items(i).Quantity > 0 then
				OptionGroups = IIf(OptionGroups = "", m_items(i).OptionGroupsList, OptionGroups & "," & m_items(i).OptionGroupsList)
			End If
		Next
	End Property
	
	Public Property Get Options
		For i = 0 to m_lastindex
			If m_items(i).Quantity > 0 then
				Options = IIf(Options = "", m_items(i).OptionsList, Options & "," & m_items(i).OptionsList)
			End If
		Next
	End Property		
	
	'----------------------------------------------------------
	'TOTALS
	'Calculates basket totals. The function 'CalculateTotals' 
	'has to be called whenever a change occurs. This loops through
	'all basket items and sets the values
	'----------------------------------------------------------
	Public Sub CalculateTotals
		'Set intial settings
		m_allfreeshipping = true
		m_hasfreeshipping = false
		m_totalitems = 0
		m_totalweight = 0
		m_totalextax = 0
		m_totaltaxamount = 0
		m_totalamountsaved = 0
		m_shippingtotalweight = 0
		m_shippingtotalextax = 0
		m_shippingtotaltaxamount = 0		
		
		'Loop through all items in the basket
		For i = 0 to m_lastindex
			With m_items(i)
				'Are all the items free-shipping?
				m_allfreeshipping = m_allfreeshipping and .FreeShipping
				
				'Is at least one item got free shipping?
				m_hasfreeshipping = m_hasfreeshipping or .FreeShipping
								
				'Totals
				m_totalitems = m_totalitems + .Quantity
				m_totalweight = m_totalweight + .RowWeight
				m_totalextax = m_totalextax + .RowExTax
				m_totaltaxamount = m_totaltaxamount + .RowTaxAmount
				m_totalamountsaved = m_totalamountsaved + .RowAmountSaved
				
				'Shipping Totals
				If Not .FreeShipping Then
					m_shippingtotalweight = m_shippingtotalweight + .RowWeight
					m_shippingtotalextax = m_shippingtotalextax + .RowExTax
					m_shippingtotaltaxamount = m_shippingtotaltaxamount + .RowTaxAmount	
				End If
			End With
		Next
		
	End Sub
	
	
	'----------------------------------------------------------
	'CALCULATE CHECKOUT TOTALS
	'This function deals with calculating all the totals required
	'during checkout - taking in to account, shipping, discounts
	'order handling etc
	'----------------------------------------------------------
	Public Sub CalculateCheckoutTotals(objRecordSetShipping, objRecordSetMisc, numShipMethod, numShipCountry, strShipPostcode, strShipISO, blnEUVat, strCouponType, numCouponValue, numCustomerDiscount)
		
		'----------------------------------------------------------	
		'PULL OUT SHIPPING DETAILS
		'----------------------------------------------------------	
		'Pull out all shiping rates - we'll use this later
		If (ListSafe(GetAppVar("shippingsystem"), "none,ups", "") <> "") Or (numShipMethod = -1 and GetAppVar("pickupoption") = "y") Or (AllFreeShipping) Then
			'If shipping is turn off/ups, the user selected local pickup, or the whole basket is
			'free from shipping, then just pull out the tax rate
			strQuery3 = "SELECT D_Tax FROM tblCactuShop" & TABLE_PREFIX & "Destination WHERE D_ID=" & C_ShippingCountry
		Else
			'Otherwise we pull out all the shipping rates in this method for this country
			'We'll use it later for shipping calculations
			strQuery3 = "SELECT * FROM (tblCactuShop" & TABLE_PREFIX & "Destination INNER JOIN tblCactuShop" & TABLE_PREFIX & "ShippingRates ON tblCactuShop" & TABLE_PREFIX & "Destination.D_ShippingZoneID = tblCactuShop" & TABLE_PREFIX & "ShippingRates.S_ShippingZoneID) INNER JOIN tblCactuShop" & TABLE_PREFIX & "ShippingMethods ON tblCactuShop" & TABLE_PREFIX & "ShippingMethods.SM_ID = tblCactuShop" & TABLE_PREFIX & "ShippingRates.S_ShippingMethodID WHERE SM_Live = 'y' AND D_ID=" & numShipCountry & " AND SM_ID=" & SQLSafe(numShipMethod) & " ORDER BY S_ShippingZoneID ASC, S_Boundary DESC"
		End If
		Call ExecuteRS(strQuery3, objRecordSetShipping)
		
		'If we didn't get a record, then we've got problems
		If objRecordSetShipping.BOF and objRecordSetShipping.EOF then 
			Call EndPage()
			response.Redirect("error.asp?" & strSessionQS & "id=badshiptaxband")
		end if
		
		'Set the basket to apply tax if there's tax in this country
		'and the we haven't got an EU vat number from the customer
		if GetAppVar("taxshippingbilling") = "billing" then
			Set objRecordSetBill = Server.CreateObject("ADODB.Recordset")
			strQuery4 = "SELECT D_Tax FROM tblCactuShop" & TABLE_PREFIX & "Destination WHERE D_ID=" & C_CardHolderCountry
			Call ExecuteRS(strQuery4, objRecordSetBill)
			ApplyTax = (objRecordSetBill("D_Tax") > 0) and not blnEUVat
			objRecordSetBill.close
			set objRecordSetBill = nothing
		else
			ApplyTax = (objRecordSetShipping("D_Tax") > 0) and not blnEUVat
		end if
		
	
		'-----------------------------------------------
		'LOOKUP SHIPPING METHOD DETAILS
		'-----------------------------------------------
		If GetAppVar("shippingsystem") = "none" then
			m_shippingType = SHIPPING_TYPE_DISABLED

		ElseIf AllFreeShipping then
			'Get the electronic/free shipping method, ls #1510, 1511
			m_shippingType = SHIPPING_TYPE_FREE
			m_shippingName = GetString("ContentText_ShippingElectronic")
			m_ShippingDescription = GetString("ContentText_ShippingElectronicDesc")

		elseif numShipMethod = -1 and GetAppVar("pickupoption") = "y" then
			'If numShipMethod = 0 then we're doing a pickup - ls #1408, 1409
			m_shippingType = SHIPPING_TYPE_PICKUP
			m_shippingName = GetString("ContentText_ShippingPickup")
			m_shippingDescription = GetString("ContentText_ShippingPickupDesc")			
			
		else
			if GetAppVar("shippingsystem") = "cactushop" then
				'Get out the shipping method being used
				m_shippingType = SHIPPING_TYPE_CACTUS
				strQuery = "SELECT * FROM tblCactuShop" & TABLE_PREFIX & "ShippingMethods WHERE SM_Live = 'y' AND SM_ID=" & NumSafe(numShipMethod)
				Call ExecuteRS(strQuery, objRecordSetMisc)
				If objRecordSetMisc.BOF and objRecordSetMisc.EOF then
					objRecordSetMisc.close
					Call EndPage()
					response.Redirect("error.asp?" & strSessionQS & "id=badshipping")
				Else	
					m_shippingName = objRecordSetMisc("SM_Name" & CInt(numLanguageID))
					m_shippingDescription = objRecordSetMisc("SM_Description" & CInt(numLanguageID))
					objRecordSetMisc.close
				End If
				
			elseif GetAppVar("shippingsystem") = "ups" then
				'Create the UPS object and set config settings
				m_shippingType = SHIPPING_TYPE_UPS
				Dim objShipping, strShopRegion
				Set objShipping = new UPSShipping
				strShopRegion = GetAppVar("UPS_ShopRegionCode")
				Call objShipping.Build(strShipISO, strShipPostcode, TotalWeight)

				'Get specific shipping rate
				objShipping.GetRate(numShipMethod)
				If objShipping.TotalShippingRates > 0 then
					'Grab the name of the shipping type from the database
					strServiceCode = CStr(numShipMethod)
					If len(strServiceCode) = 1 then strServiceCode = "0" & strServiceCode
					strQuery = "SELECT UPS_Name FROM tblCactuShop" & TABLE_PREFIX & "UPS WHERE UPS_Region = " & strUnicode & "'" & SQLSafe(strShopRegion) & "' AND UPS_Code = " & strUnicode & "'" & SQLSafe(strServiceCode) & "'"
					Call ExecuteRS(strQuery, objRecordSetMisc)
					If not (objRecordSetMisc.BOF and objRecordSetMisc.EOF) then
						numUPSShippingCost = objShipping.ShippingRates(1).Cost
						m_shippingName = "UPS"
						m_shippingDescription = objRecordSetMisc("UPS_Name")
						objRecordSetMisc.Close
					Else
						objRecordSetMisc.Close
						Call EndPage()
						response.Redirect("error.asp?" & strSessionQS & "id=upsnotindb&code=" & strServiceCode)
					end if
				end if
			end if
		end if

		
		'-----------------------------------------------
		'CALCULATE DEDUCTIONS, TOTALS AND SHIPPING
		'-----------------------------------------------		
		'Set the customer discount and zero shipping
		m_customerDiscountPercentage = NumSafe(numCustomerDiscount)
		blnZeroShipping = (m_shippingType = SHIPPING_TYPE_DISABLED or m_shippingType = SHIPPING_TYPE_PICKUP or m_shippingType = SHIPPING_TYPE_FREE)
		
				
		'Calculation various order modifiers - order matters here
		'as the discount total updates as we go - important for %
		'discounts 
		Call CalculatePromotions(objRecordSetMisc, false, false)
		Call CalculateCoupon(strCouponType, numCouponValue, false)
		Call CalculateCustomerDiscount(m_customerDiscountPercentage)
		Call CalculateOrderHandlingCharge(objRecordSetMisc)

		If GetAppVar("pricesinctax") <> "y" then numShippingTotalAmount = ShippingTotalExTax else numShippingTotalAmount = ShippingTotalIncTax
		Call CalculateShipping(objRecordSetShipping, objRecordSetMisc, blnZeroShipping, numShippingTotalAmount, numUPSShippingCost)
		
		'Close shipping recordset
		objRecordSetShipping.Close

	End Sub
	

	'----------------------------------------------------------
	'GET VERSION QUANTITY
	'Gets the quantity in the basket for a specific version.
	'Note that we can have more one entry for the same version
	'(one product with multiple options)
	'----------------------------------------------------------
	Private Function GetVersionQuantity(V_ID)
		numVersion = NumSafe(V_ID)
		GetVersionQuantity = 0
		If numVersion > 0 then
			For numBasketCount = 0 to Count - 1
				If Item(numBasketCount).VersionID = V_ID then
					'Got an index match, exit
					GetVersionQuantity = GetVersionQuantity + Item(numBasketCount).Quantity
				End If
			Next
		End If
	End Function
	
	'Gets the lowest price (inc and ex tax) for a version in
	'the basket. For the reason reason as above, there could
	'be more than once price to pick from
	Private Function GetLowestPrice(V_ID, blnIncTax)
		numVersion = NumSafe(V_ID)
		GetLowestPrice = 0
		If numVersion > 0 then
			For numBasketCount = 0 to Count - 1
				If Item(numBasketCount).VersionID = V_ID then
					'Got an index match
					If GetLowestPrice = 0 then
						If blnIncTax then
							GetLowestPrice = Item(numBasketCount).IncTax
						Else
							GetLowestPrice = Item(numBasketCount).ExTax
						End If
					Else
						If blnIncTax then
							GetLowestPrice = Min(GetLowestPrice, Item(numBasketCount).IncTax)
						Else
							GetLowestPrice = Min(GetLowestPrice, Item(numBasketCount).ExTax)
						End If
					End If
				End If
			Next
		End If
	End Function 
	
	
	
	'-----------------------------------------------
	'CALCULATE PROMOTIONS
	'Calculates the promotional discounts based on
	'products in the basket.
	'-----------------------------------------------
	Public Sub CalculatePromotions(objRS, blnLinkToVersions, blnZeroTotalTaxRate)

		If SOFTWARE_IS_PRO then
			'Pull out all promotions related to our checkout basket
			Set objRS2 = Server.CreateObject("ADODB.RecordSet")
			Set objPromotionInfo = New PromotionGroup

			'Get list of items in the basket (we don't want to use the Items function,
			'as that doesn't return the combination IDs which we need to this
			strComboItems = GetComboItems(m_items, m_lastindex)
			Call objPromotionInfo.BuildPromotionsForCheckout(objRS, objRS2, strComboItems, blnLinkToVersions)
			
			'Set our array of promotions to the right size
			Redim Promotions(objPromotionInfo.Count)
			
			'Loop through all promotions while we haven't met the maximum
			i = 0
			numAppMaxPromotions = GetAppVar("maxpromotions")
			
			Do While i <= (objPromotionInfo.Count - 1) And (PromotionsCount < numAppMaxPromotions or numAppMaxPromotions = 0)

				Set Promotions(PromotionsCount+1) = New PromotionBasketModifier
				With Promotions(PromotionsCount+1)

					'Set up the promotion basket modifier from the available promotions
					Call .SetFromPromotion(objPromotionInfo.Item(i), m_items, m_lastindex, IIf(blnZeroTotalTaxRate, 0, TotalTaxRate), TotalIncTax)
					
					'Work out the running totals by multiplying the total with the quantity
					If .Quantity > 0 then

						'Work back to work out the tax rate
						.TaxRate = TotalDiscountPriceTaxRate
					
						'Round the value either by item or by row depending in inctax setting
						If PricesIncTax then
							.IncTax = NegSafeRound(.IncTax, 2)
							.ExTax = NegSafeRound(.ExTax, 2)
							.TotalIncTax = .IncTax * .Quantity
							.TotalExTax = .ExTax * .Quantity
						Else
							.TotalIncTax = NegSafeRound(.IncTax * .Quantity, 2)
							.TotalExTax = NegSafeRound(.ExTax * .Quantity, 2)
						End If
						
						'Update our running totals
						numPromotionDiscountIncTax = numPromotionDiscountIncTax + .TotalIncTax
						numPromotionDiscountExTax = numPromotionDiscountExTax + .TotalExTax
						
						'Increment count if we've got quantity. If we haven't, then by not
						'incrementing here, we overwrite it
						PromotionsCount = PromotionsCount + 1	
						
					End If
				End With
				i = i + 1
			Loop
			
			'Negate the promotion totals and calculate tax ammount. These are stored
			'in global settings in the basket
			PromotionDiscount.TaxRate = TotalDiscountPriceTaxRate
			PromotionDiscount.IncTax = numPromotionDiscountIncTax
			PromotionDiscount.ExTax = numPromotionDiscountExTax
		
			Set objRS2 = Nothing	
		
		Else
			
			PromotionDiscount.TaxRate = 0
			PromotionDiscount.IncTax = 0
			PromotionDiscount.ExTax = 0
		
		End If
	End Sub
	
	'-----------------------------------------------
	'COUPONS DISCOUNT
	'-----------------------------------------------
	Public Sub CalculateCoupon(strCouponType, numCouponValue, blnZeroTotalTaxRate)
		CouponDiscount.TaxRate = TotalDiscountPriceTaxRate
		Select Case strCouponType
			Case "p"
				'Percentage coupons
				CouponDiscount.IncTax = -SafeRound((numCouponValue * TotalDiscountPriceIncTax / 100), 2)
				CouponDiscount.ExTax = -SafeRound((numCouponValue * TotalDiscountPriceExTax / 100), 2)
				
			Case "f"
				'Fixed rate coupons
				If numCouponValue > TotalDiscountPriceExTax then
					CouponDiscount.ExTax = -TotalDiscountPriceExTax
					CouponDiscount.IncTax = -TotalDiscountPriceIncTax
				Else
					CouponDiscount.IncTax = -SafeRound(numCouponValue * objCurrency.ExchangeRate, 2)
					
					If blnZeroTotalTaxRate then
						CouponDiscount.ExTax = NegSafeRound(CouponDiscount.IncTax, 2)
					Else
						CouponDiscount.ExTax = NegSafeRound(CouponDiscount.IncTax * (1 - (CouponDiscount.TaxRate / (1 + TotalTaxRate))), 2)
					End If
				End If
				
			Case Else
				'Dodgy coupon - ignore
				CouponDiscount.IncTax = 0
				CouponDiscount.ExTax = 0
		end Select
	End Sub
	
	
	'-----------------------------------------------
	'CALCULATE SHIPPING
	'Based on the customers shipping method selection
	'-----------------------------------------------	
	Public Sub CalculateShipping(objRecordSetShipping, objRecordSetMisc, blnZeroShipping, numTotalToShip, numUPSShippingCost)
		
		'Look up shipping tax rate
		strQuery = "SELECT T_Taxrate FROM tblCactuShop" & TABLE_PREFIX & "TaxRates WHERE T_ID=" & NumSafe(GetAppVar("shiptaxband"))
		Call ExecuteRS(strQuery, objRecordSetMisc)
		If objRecordSetMisc.BOF and objRecordSetMisc.EOF then
			objRecordSetMisc.Close
			Call EndPage()
			response.Redirect("error.asp?" & strSessionQS & "id=badshiptaxband")
		Else
			if objBasket.D_Tax <> "" then shippingPrice.TaxRate = (objRecordSetMisc("T_Taxrate") / 100) * objBasket.D_Tax	else shippingPrice.TaxRate = (objRecordSetMisc("T_Taxrate") / 100)
			
			objRecordSetMisc.close
		End If
		
		'Calculate shipping costs
		strAppShippingSystem = GetAppVar("shippingsystem")
		If blnZeroShipping then
			'Shipping disabled
			numShippingPriceValue = 0
			
		ElseIf strAppShippingSystem = "cactushop" then
			'Loop through all the shipping boundaries
			do while not objRecordSetShipping.EOF
				if GetAppVar("calcshipcostbyweight") = "y" then
					If objRecordSetShipping("S_boundary") >= ShippingTotalWeight then
						numShippingPriceValue = objRecordSetShipping("S_ShippingRate")
					end if
				else
					If objRecordSetShipping("S_boundary") >= (numTotalToShip / objCurrency.ExchangeRate) then
						numShippingPriceValue = objRecordSetShipping("S_ShippingRate")
					end if
				end if	
				objRecordSetShipping.moveNext
			loop
			
			'Convert to the users currency
			numShippingPriceValue = SafeRound(numShippingPriceValue * objCurrency.ExchangeRate, 2)
			

		elseif GetAppVar("shippingsystem") = "ups" then
			'If we are including tax in display, shipping price should include all tax
			If PricesIncTax then
				numShippingPriceValue = numUPSShippingCost * GetAppVar("UPS_CurrencyConversion") * objCurrency.ExchangeRate
				numShippingPriceValue = SafeRound(numShippingPriceValue * (1 + ShippingPrice.TaxRate), 2)
			else
				'otherwise leave without and the ex-tax can be displayed
				numShippingPriceValue = SafeRound(numUPSShippingCost * GetAppVar("UPS_CurrencyConversion") * objCurrency.ExchangeRate, 2)
			end if
		end if
				
		'Set shipping inc and ex tax values
		If PricesIncTax then
			'Set the extax and inctax shipping values
			ShippingPrice.ExTax = SafeRound(numShippingPriceValue * (1 / (1 + ShippingPrice.TaxRate)), 2)
						
			'If tax is off, then inc tax can be set to just the ex tax
			If ApplyTax then
				ShippingPrice.IncTax = numShippingPriceValue
			Else
				ShippingPrice.IncTax = ShippingPrice.ExTax
			End if
		Else
			'Tax rate for shipping
			If Not ApplyTax then
				ShippingPrice.TaxRate = 0
			End If
			
			'Set the extax and inctax shipping values
			ShippingPrice.ExTax = numShippingPriceValue
			ShippingPrice.IncTax = SafeRound(ShippingPrice.ExTax * (1 + ShippingPrice.TaxRate), 2)
			
		End if
		
	End Sub	
	
	
	'--------------------------------------
	'ORDER HANDLING CHARGE
	'--------------------------------------
	Public Sub CalculateOrderHandlingCharge(objRecordSetMisc)
		
		'Get the value and the tax band
		numOrderHandlingPriceValue = GetAppVar("orderhandlingcharge") * objCurrency.ExchangeRate
		numOrderHandlingTaxBand = GetAppVar("orderhandlingchargetaxband")

		'Get the tax rate
		strQuery = "SELECT T_Taxrate FROM tblCactuShop" & TABLE_PREFIX & "Destination, tblCactuShop" & TABLE_PREFIX & "TaxRates WHERE D_ID=" & C_ShippingCountry & " AND T_ID =" & numOrderHandlingTaxBand
		Call ExecuteRS(strQuery, objRecordSetMisc)
		If objRecordSetMisc.BOF and objRecordSetMisc.EOF then
			objRecordSetMisc.Close
			Call endPage()
			response.redirect("error.asp?" & strSessionQS & "id=badhandlingtaxband")
		Else
			if objBasket.D_Tax <> "" then OrderHandlingPrice.TaxRate = (objRecordSetMisc("T_Taxrate") / 100) * objBasket.D_Tax else OrderHandlingPrice.TaxRate = objRecordSetMisc("T_Taxrate") / 100
			objRecordSetMisc.Close
		End If

		If PricesIncTax then
			'Set the extax and inctax order handling values
			OrderHandlingPrice.ExTax = SafeRound(numOrderHandlingPriceValue * (1 / (1 + OrderHandlingPrice.TaxRate)), 2)
			
			'If tax is off, then inc tax can be set to just the ex tax
			If ApplyTax then
				OrderHandlingPrice.IncTax = numOrderHandlingPriceValue
			Else
				OrderHandlingPrice.IncTax = OrderHandlingPrice.ExTax
			End if
		Else
			'Tax rate for order handling
			If Not ApplyTax then
				OrderHandlingPrice.TaxRate = 0
			End If
			
			'Set the extax and inctax order handling values
			OrderHandlingPrice.ExTax = numOrderHandlingPriceValue
			OrderHandlingPrice.IncTax = SafeRound(OrderHandlingPrice.ExTax * (1 + OrderHandlingPrice.TaxRate), 2)
		End if
	End Sub	
	
	'--------------------------------------
	'ORDER HANDLING CHARGE
	'--------------------------------------
	Public Sub CalculateCustomerDiscount(numCustomerDiscount)
		CustomerDiscount.TaxRate = TotalDiscountPriceTaxRate
		CustomerDiscount.IncTax = -SafeRound(TotalDiscountPriceIncTax * (numCustomerDiscount / 100), 2)
		CustomerDiscount.ExTax = -SafeRound(TotalDiscountPriceExTax * (numCustomerDiscount / 100), 2)
	End Sub
End Class

%>
